/********************************************************************************
	T/SDAS-SCOM2 DCM Analog I/O Stand-alone Demo/Test Utility for C54xx DSP. v.2.0
	(for all TORNADO-54xx/E54xx DSP Systems/Controllers)

	(C) MicroLAB Systems Ltd, 1994-2003

	WEB:    www.mlabsys.com
	FTP:    ftp://ftp.mlabsys.com/sft
	e-mail: techsupport@mlabsys.com


	Description:
	------------
	  1. This demo runs on TORNADO-54xx DSP Systems and TORNADO-E6x DSP Controllers
		 within DSP environment only in stand-alone mode and does not require
		 host control.
	  2. This DSP demo has been designed to be user configured by means of
		 compiler run-time definitions inside SCOM2_C6X.CFG file, that allows to
		 select different demo modes and parameters:
			- TORNADO platform selection (either TORNADO-54xx, or TORNADO-E6x)
			- run-time configuration parameters (sampling frequency and signal bandwidth,
			  analog input mux, analog input gain and analog output gain)
	  3. MIRAGE-510DX/P510D JTAG emulator and TI C5000 CCS tools are required to
		 recompile this demo and to upload code to the DSP environment.
	  4. It is possible to debug DSP code using JTAG emulator.
	  5. DSP on-chip BSP0 (TORNADO-548/549/E548/E549) or McBSP0 (TORNADO-5402/5410/5416/E5402/E5410/E5416)
	     serial port is used for communication with T/SDAS-SCOM2 DCM, i.e. DCM is assumed to be instaled to on-board
		 SIOX-A rev.B site. Detection of the type for TORNADO on-board DSP is performed
		 automatically in order to use either BSP serial port (TMS320C548/C549 DSP) or
		 McBSP0 (TMS320C5402/C5410/C5416) serial port.
	  6. T54x/E54x XIO-0/1 I/O pins can be be configured as OUTPUT and is used as
		 optional performance tracking output.
	  7. This demo sets all SCOM2 DOUT-0..3 digital outputs to the '1' state in order to
	     allow external connection and activation of XTLI option for each AD/DA channel, thus
	     putting it to the OFF HOOK state. In this mode, XTLI option, once connected to
	     any channel of SCOM2 DCM, will be able to operate with PSTN line. For example,
		 the SINE output test procedure for SCOM2+XTLI can be the following:
	     	- load SINE output demo for your TORNADO-54x or TORNADO-E54xx board. When loading via
			  JTAG emulator and TI C5000 CCS tools, be sure to set the DSP on-chip OVLY bit into 
			  the '1' state via CCS REGISTERS window.
	     	- connect external XTLI option via dual-channel splitter to any channel
	     	  of SCOM2 DCM. Be sure to provide extrenal +12v, -12v, and -5v to TORNADO-E54xx DSP
	     	  controllers in this case.
	     	- provide PSTN phone device with parallel PSTN socket extender connected to the PSTN
	     	- connect PSTN cable into any I/O PSTN connector of XTLI option and, on the other side,
			  disconnect this cable from the PSTN (unplug it from the PSTN socket, which is parallel
			  to the PSTN phone
			- run the SINE output demo
			- make a call to the PSTN phone from another phone
			- when the phone will ring, pick-up the phone
			- connect the cable from XTLI to the PSTN socket, which is parallel to the PSTN phone
			- you must hear SINE signal in the line from the calling phone
			- you can now disconnect the PSTN phone from its PSTN socket, since XTLI option will
			  hold the line
		 
		 Note, that no code is provided for detection of incoming RING call and automatic OFF HOOK
		 processing for XTLI. However this code can be easily added to this demo.

	     As default, external XTSI option is not directly supported in this demo as it is, since XTLI
		 option requires synchronization of digital intputs and digital outputs, which are used to 
		 control it.  However, XTSI support code can be easily added to this demo.

	  8. The demo performance is as it is due to non-optimized demo code.


	T/SDAS-SCOM2 daughter-card module installation:
	--------------------------------------------------------
	  1. T/SDAS-SCOM2 DCM must be installed into SIOX-A site at TORNADO mainboard.


	Loading Demo and Initialization:
	---------------------------------
		All demos can be loaded into DSP environment via MIRAGE JTAG emulator
		(or compatible).


  	Copyright:
  	----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


*********************************************************************************/

// general includes
#include	"math.h"
#include	"stdlib.h"

// application specific include files
#include	"c54x_def.h"	// include C54x DSP definitions


// DSP master control descriptor (MCD)
// MCD is actually not required for this demo and can be just replaced by a set of
// demo parameters. However it has been enclosed for providing compatibility with
// host PC based demos for TORNADO+DCM, which used MCD to pass demo parameters from
// host PC to the TORNADO on-board DSP and to monitor demo run-time status.

struct	MCD_DD
{
	int 			status;		    // Master STATUS: MSTAT_RDY/BUSY/ATTN
   	int 			err;        	// command exec error

   	int 			cmnd;   	    // command to exec
   	unsigned 		epoch_len;      // epoch length

   	unsigned 		aic_ra,aic_rb;  // AIC RA/RB on-chip dividers
  	unsigned 		aic_ain_mux;    // AIC analog input MUX cgf (0 - main AIN, 1- AUX AIN)

   	unsigned 		aic_ain_gain;	// AIC analog input gain/scale
   	unsigned 		aic_aout_att;	// AIC analog output attenuation/scale

	int				mcbsp_flag;		// ==0 for C54x with BSP port (C542/C548/C549)
									// ==1 for C54xx with McBSP ports (C5402/C5410/C5416)
};

// default address for MCD in D-SRAM is *(0xff00)
struct	MCD_DD *MCD	= (struct MCD_DD *)0xff00;


// Master status values
#define	MSTAT_RDY					0
#define	MSTAT_BUSY					1
#define	MSTAT_ATTN					2
#define	MSTAT_GO					3
#define	MSTAT_ABORT					4

// Master returned error codes
#define	MERR_OK						0
#define	MERR_INV_CMND				1 	//     ; invalid command
#define	MERR_INV_ATTN				2
#define	MERR_HW_ERR					3
#define	MERR_SMALL_MEMORY_ERR		4
#define	MERR_INV_CNF_ERR			5

// Master commands defs
#define	MCMND_NOP					0
#define	MCMND_ABORT					1

#define	MCMND_DAC_SINE_OUTPUT		0x102
#define	MCMND_ADC_TO_DAC			0x103


// general defs
#define	ML_OFF						0
#define	ML_ON						1


// maximum data array length for one ADC/DAC channel (in samples)
// ADC/DAC data bufers are allocated fm heap, which is in external memory
// (heap length is set to 0x7f00 in order to fit all TMS320C54xx DSP)
#define	HEAP_LEN					0x1000
#define	DATA_BUF_LEN				((HEAP_LEN/2)/2) 	// maximum data array length for each of IN/OUT channel

// global vars
int	 *ADC=NULL,*DAC=NULL;			// ptrs to the ADC/DAC data

#define	SIO_NOT_MCBSP				((MCD->mcbsp_flag)==0)
#define	SIO_MCBSP					((MCD->mcbsp_flag)==1)


//---------------------- include DEMO CNF file ---------------------------
#include	"scom2_c54x.cfg"	// include run-time compiler defs for demo mode and params


	// perform TORNADO-E54x specific definitions and include
#if(_E54X_SELECTED)
	// include defs for E54x
	#include	"te54x_d.h"

	// define XIO-flags control macros
	#define	_SET_XIOF1_OFF_		TE54X_SET_XIO1_DATA_OFF
	#define	_SET_XIOF1_ON_		TE54X_SET_XIO1_DATA_ON
	#define	_SET_XIOF0_OFF_		TE54X_SET_XIO0_DATA_OFF
	#define	_SET_XIOF0_ON_		TE54X_SET_XIO0_DATA_ON

	// define SIOX/PIOX RESET release macros
	#define	_REMOVE_DCM_RESET	TE54X_set_pxsx_reset_rg(0x7)	// release SIOX-A/B and PIOX reset
	#define	_SET_DCM_RESET		TE54X_set_pxsx_reset_rg(0)		// set SIOX-A reset

#endif


	// perform TORNADO-54x specific definitions and include
#if(_T54X_SELECTED)
	#include	"t54x_dsp.h"	// include defs for T54x

	// define XIO-flags control macros
	#define	_SET_XIOF1_OFF_		T54X_set_xiof1(0)
	#define	_SET_XIOF1_ON_		T54X_set_xiof1(1)
	#define	_SET_XIOF0_OFF_		T54X_set_xiof0(0)
	#define	_SET_XIOF0_ON_		T54X_set_xiof0(1)

	// define SIOX/PIOX RESET release macros
	#define	_REMOVE_DCM_RESET	T54X_set_pxsx_run(0x7)			// release SIOX-A/B and PIOX reset
	#define	_SET_DCM_RESET		T54X_set_pxsx_run(0)			// set SIOX-A reset
#endif

//-----------------------------------------------------------------------

// include T/SDAS-SCOM2 application specific DSP code
#include	"scom2_c54x.i"





#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
// include DAC SINE fill-in function in case DEMO FUNCTION is DAC_SINE_OUTPUT

#define	M_PI	3.141592

void fill_dac_sine()
// fill-in DAC buffers with sine data with different frequency for each channel
{
	int *D;
    unsigned  chan_count,n;

    for (chan_count=0,D=DAC; chan_count<2; chan_count++)
    {
		for(n=0;n<DATA_BUF_LEN;n++)
		{
			switch(chan_count)
			{
				case 0:
			    	*D++ = (int) ((float) (5000.0*sin(50.0*M_PI*((float)n)/((float) DATA_BUF_LEN))));
			      	break;

			   	case 1:
			     	*D++ = (int) ((float) (5000.0*sin(100.0*M_PI*((float)n)/((float) DATA_BUF_LEN))));
			     	break;
			}
		}
    }
}
#endif




cpu_init()
// performs DSP initialization and detection of the DSP type for TORNADO-54x DSP systems
// in order to detect type of DSP on-chip serial ports (BSP or McBSP)
{
 	// clear IMR since it is not cleared on reset
	C54X_IMR_CLEAR;
	C54X_IFR_CLEAR;

	// perform platform specific initialization

		// perform TORNADO-E54x DSP System initialization
#if(_E54X_SELECTED)
	// initialize DSP and assume that external DATA-RAM is active
	TE54X_init_for_xdm_dram();

    // configure XIO-0/1 as OUTPUT pins
  	TE54X_SET_XIO1_CNF_IO;
	TE54X_SET_XIO0_CNF_IO;
  	TE54X_SET_XIO1_DIR_OUTPUT;
  	TE54X_SET_XIO0_DIR_OUTPUT;

	// set mcbsp_flag in accordance with the E54xx board type
	if(TE548_BOARD || TE549_BOARD)
		MCD->mcbsp_flag =  0;
	else
		MCD->mcbsp_flag =  1;
#endif

		// perform TORNADO-54x DSP System initialization
#if(_T54X_SELECTED)
	// initialize DSP
  	T54X_init_dsp();

    // configure XIO-0/1 as OUTPUT pins
	T54X_set_xio0_cnf(T54X_XIO_IO_PIN);
	T54X_set_xio1_cnf(T54X_XIO_IO_PIN);
  	T54X_set_xiof1_dir(T54X_XIOF_OUTPUT);
  	T54X_set_xiof0_dir(T54X_XIOF_OUTPUT);

	// perform recognition between BSP/McBSO DSP on-chip serial ports
	// Detection is done via the check of presence of 11-bit ARR/BKR registers for BSP0
	// serial port at 0x3A/0x3B addresses, which present in C542/C548/C549 DSP only
	// For C5402/C5410/C5416 DSP with McBSP serial ports, addresses 0x3A/0x3B are not used
	// for memory mapped registers and shall either always read as 0xffff or undefined.
		// assume initially that the McBSP port has been detected
	MCD->mcbsp_flag =  1;

    	// write two different 16-bit patterns to ARR/BKR registers of BSP0 port
    	// read and verify returned data: if returned data is 11 bits of the written data,
		// then this is the BSP port of C548/C549 DSP
    *C54X_BSP_ARR = 0xaaaa;
    *C54X_BSP_BKR = 0x5555;
    if(		(*C54X_BSP_ARR) == (0xaaaa & 0x7ff)
    	||	(*C54X_BSP_BKR) == (0x5555 & 0x7ff)
      )
    {
	    *C54X_BSP_ARR = 0xffff;
    	*C54X_BSP_BKR = 0xaa55;
    	if(		(*C54X_BSP_ARR) == (0xffff & 0x7ff)
    		||	(*C54X_BSP_BKR) == (0xaa55 & 0x7ff)
    	  )
			MCD->mcbsp_flag =  0;	// after all, this is the BSP port of C542/C548/C549 DSP
    }
#endif


	// reset DCM in order to reset synchronization in case application has been
	// restarte several times without DSP hardware reset.
	_SET_DCM_RESET;
	_wait_for_dcm_reset();

	// clear TM/XIO-0/1 flags if they are used for time tracking
	_SET_XIOF0_OFF_;
	_SET_XIOF1_OFF_;

	return(0);
 }




int	cmnd_decoder()
// decodes command
 {
	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
       	// application specific commands
	   case MCMND_DAC_SINE_OUTPUT:
	   case MCMND_ADC_TO_DAC:
	   	 return( daq_run() );

		// all unknown commands cause error
	   default:
		 return( MERR_INV_CMND );
	 }
 }






void main()
{
	// first, disable IRQ if this is the program restart under the emulator without DSP reset
	C54X_INT_DISABLE;

	// allocate ADC/DAC buffers fm the heap and init CPU
	if(     (ADC=(int *) malloc(DATA_BUF_LEN*2))==NULL
	   || 	(DAC=(int *) malloc(DATA_BUF_LEN*2))==NULL
	   ||	cpu_init()!=MERR_OK
	  )
		goto error;

	// set MCD params from the .CFG file
	MCD->epoch_len		= DATA_BUF_LEN;		// set AD/DA buffer length
	MCD->status			= MSTAT_RDY;

	MCD->cmnd			= _FUNC_;
	MCD->aic_ra			= _AIC_RA_;
	MCD->aic_rb			= _AIC_RB_;
	MCD->aic_ain_mux	= _AIC_AIN_MUX_;
	MCD->aic_ain_gain	= _AIC_AIN_GAIN_;
	MCD->aic_aout_att	= _AIC_AOUT_ATT_;


	// fill-in DAC data buffers with SINE data for DAC SINE OUTPUT demo function
#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
	fill_dac_sine();
#endif

	// call exec & return error code
	MCD->err = cmnd_decoder();

	// clear status flag
    MCD->status = MSTAT_RDY;


	// get here and idle infinitely on any error condition
error:
	while(1);
}

